// California Individual-based Fish Simulation System. 
// Copyright (C) 1999 Lang Railsback & Assoc.
// This library is distributed without any warranty; without even the
// implied warranty of merchantability or fitness for a particular purpose.
// See file LICENSE for details and terms of copying.

#import "ExperBatchSwarm.h"

@protocol Model

- (id <Zone>) getModelZone;

@end

@protocol Object

- (id <Zone>) getObjectZone;

@end

@implementation ParameterBatchManager

- (BOOL) canWeGoAgain {

  return (modelIterator++ != 50);

}


@end


@implementation ExperBatchSwarm

+ createBegin: aZone
{
  ExperBatchSwarm *obj;

  obj = [super createBegin: aZone];

  obj->numExperimentsRun = 0;

  //xprint(aZone);

  return obj;		// We return the newly created ExperBatchSwarm
}


- createEnd
{
  return [super createEnd];
}



- buildObjects
{

  [super buildObjects];

  // Build the parameter manager

  parameterManager = [ParameterBatchManager create: self];

  return self;
}

- buildActions
{
  [super buildActions];

  experSchedule = [Schedule createBegin: self];
  [experSchedule setRepeatInterval: 1];
  experSchedule = [experSchedule createEnd];

  [experSchedule at: 0 createActionTo: self	message: M(setupModel)];
  [experSchedule at: 0 createActionTo: self	message: M(buildModel)];
  [experSchedule at: 0 createActionTo: self	message: M(runModel)];
  [experSchedule at: 0 createActionTo: self     message: M(checkToStop)];
  [experSchedule at: 0 createActionTo: self	message: M(dropModel)];


  return self;
}  

- activateIn: swarmContext
{
  [super activateIn: swarmContext];
  [experSchedule activateIn: self];
  return [self getActivity];
}



///////////////////////////////////////////////////
//
// setupModel
//
///////////////////////////////////////////////////
- setupModel
{


  //fprintf(stderr, "EXPERBATCHSWARM >>>>>> Setup Model\n");
  //fflush(0);

  subSwarm = [TemplateBatchSwarm create: self];

  [subSwarm objectSetup];

  return self;

}


- buildModel {
  //fprintf(stderr, "EXPERBATCHSWARM >>>>>> Build Model\n");
  //fflush(0);

  [subSwarm buildObjects];

  [subSwarm buildActions];

  [subSwarm activateIn: nil];

  return self;
}  




////////////////////////////////////////////////////////////
//
// runModel
//
////////////////////////////////////////////////////////////
- runModel
{
  static id <Symbol> activityState;
  BOOL doneRunning=NO;

  // We have built the model and activated it - here is where we run it.
  // When it has terminated, control will return here.

  while (((activityState = [[subSwarm getActivity] run]) != Terminated) &&
	 (activityState != Stopped))
            
                   ;  // just loop

  //printf("\nactivityState = %s \n", [activityState getName]);

  if ((doneRunning = [subSwarm areYouFinishedYet]) == YES) {
    
 
    numExperimentsRun++;               // increment count of models


  }
  else if (doneRunning == NO) {
  
 
  }


  return self;
}


- dropModel
{


  [[subSwarm getActivity] drop];
  [subSwarm drop];
  subSwarm = nil; 

  
/*
  fprintf(stderr,"<<DROPMODEL>>\n");
  fflush(0);
  xprint(globalZone);
  fprintf(stderr,"<<DROPMODEL>>\n");
  fflush(0);
*/

  return self;
}


- checkToStop
{
  // If all the models have run, time to quit!
  if ([parameterManager canWeGoAgain] == NO) {
      
      //printf("\n All the models have run!\n");
      
      [[self getSwarmActivity] stop];

    }

  return self;
}

- go {

   //fprintf(stderr, "\n You  are running in batchmode \n");
   //fflush(stderr);  

  [[self getSwarmActivity] run];

  return [[self getSwarmActivity] getStatus];

}
@end
